@@ -19,7 +19,9 @@
 #include <string.h>
 
 #include <gdk/gdkkeysyms.h>
+#ifndef _WIN32
 #include <gdk/gdkx.h>
+#endif
 
 #include <gtk/gtk.h>
 
@@ -173,7 +175,7 @@ static void add_widget_to_container_finish (GtkWidget * new_widget,
 					    GbWidgetNewData * data);
 static gint expose_widget (GtkWidget * widget,
 			   GdkEventExpose * event,
-			   GbWidgetData * wdata);
+			   GladeWidgetData * wdata);
 static gint draw_widget (GtkWidget * widget,
 			 GdkRectangle * area,
 			 gpointer data);
@@ -268,20 +270,25 @@ editor_set_show_grid (gboolean show)
 
 
 void
-editor_show_grid_settings_dialog (void)
+editor_show_grid_settings_dialog (GtkWidget *widget)
 {
-  GtkWidget *window, *vbox, *table, *label, *button, *hbbox;
+  GtkWidget *dialog, *vbox, *table, *label, *button, *hbbox;
   GtkWidget *separator, *spinbutton;
   GtkObject *adjustment;
   GSList *group;
+  GtkWidget *transient_parent;
 
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (window), _("Grid Options"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_wmclass (GTK_WINDOW (window), "grid_options", "Glade");
+  dialog = gtk_window_new (GTK_WINDOW_DIALOG);
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Grid Options"));
+  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW (dialog), "grid_options", "Glade");
+  transient_parent = glade_util_get_toplevel (widget);
+  if (GTK_IS_WINDOW (transient_parent))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				  GTK_WINDOW (transient_parent));
 
   vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
+  gtk_container_add (GTK_CONTAINER (dialog), vbox);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
   gtk_widget_show (vbox);
 
@@ -297,11 +304,12 @@ editor_show_grid_settings_dialog (void)
 
   adjustment = gtk_adjustment_new (editor_grid_horz_spacing, 1, 1000, 1,
 				   10, 10);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0, 0);
+  spinbutton = glade_util_spin_button_new (GTK_OBJECT (dialog), "spinbutton1",
+					   GTK_ADJUSTMENT (adjustment), 0, 0);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 3, 0, 1,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (spinbutton);
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton1", spinbutton);
+  gtk_widget_grab_focus (spinbutton);
 
   label = gtk_label_new (_("Vertical Spacing:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -311,11 +319,11 @@ editor_show_grid_settings_dialog (void)
 
   adjustment = gtk_adjustment_new (editor_grid_vert_spacing, 1, 1000, 1,
 				   10, 10);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0, 0);
+  spinbutton = glade_util_spin_button_new (GTK_OBJECT (dialog), "spinbutton2",
+					   GTK_ADJUSTMENT (adjustment), 0, 0);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 3, 1, 2,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (spinbutton);
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton2", spinbutton);
 
   table = gtk_table_new (1, 3, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
@@ -333,7 +341,7 @@ editor_show_grid_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 1, 2, 0, 1,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button1", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button1", button);
   group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 
   button = gtk_radio_button_new_with_label (group, _("Lines"));
@@ -342,7 +350,7 @@ editor_show_grid_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 2, 3, 0, 1,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 1);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button2", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button2", button);
 
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (vbox), separator, TRUE, TRUE, 5);
@@ -376,32 +384,35 @@ editor_show_grid_settings_dialog (void)
   gtk_widget_show (button);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
+			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
 
-  gtk_widget_show (window);
+  gtk_widget_show (dialog);
 }
 
 
 static void
 on_grid_settings_ok (GtkWidget * widget, gpointer data)
 {
-  GtkWidget *window, *spinbutton, *button;
+  GtkWidget *dialog, *spinbutton, *button;
 
-  window = gtk_widget_get_toplevel (widget);
-  spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton1");
+  dialog = gtk_widget_get_toplevel (widget);
+  spinbutton = gtk_object_get_data (GTK_OBJECT (dialog), "spinbutton1");
   editor_grid_horz_spacing = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 							       (spinbutton));
-  spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton2");
+  spinbutton = gtk_object_get_data (GTK_OBJECT (dialog), "spinbutton2");
   editor_grid_vert_spacing = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 							       (spinbutton));
 
-  button = gtk_object_get_data (GTK_OBJECT (window), "button1");
+  button = gtk_object_get_data (GTK_OBJECT (dialog), "button1");
   if (GTK_TOGGLE_BUTTON (button)->active)
     editor_grid_style = GB_GRID_DOTS;
   else
     editor_grid_style = GB_GRID_LINES;
 
-  gtk_widget_destroy (window);
+  gtk_widget_destroy (dialog);
 
   /* redraw all windows */
   glade_project_foreach_component (current_project, editor_redraw_component,
@@ -434,17 +445,22 @@ editor_set_snap_to_grid (gboolean snap)
 
 
 void
-editor_show_snap_settings_dialog (void)
+editor_show_snap_settings_dialog (GtkWidget *widget)
 {
-  GtkWidget *window, *vbox, *table, *label, *button, *hbbox, *separator;
+  GtkWidget *dialog, *vbox, *table, *label, *button, *hbbox, *separator;
+  GtkWidget *transient_parent;
 
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (window), _("Snap Options"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_wmclass (GTK_WINDOW (window), "snap_options", "Glade");
+  dialog = gtk_window_new (GTK_WINDOW_DIALOG);
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Snap Options"));
+  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW (dialog), "snap_options", "Glade");
+  transient_parent = glade_util_get_toplevel (widget);
+  if (GTK_IS_WINDOW (transient_parent))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				  GTK_WINDOW (transient_parent));
 
   vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
+  gtk_container_add (GTK_CONTAINER (dialog), vbox);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
   gtk_widget_show (vbox);
 
@@ -465,7 +481,8 @@ editor_show_snap_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 1, 2, 0, 1,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button1", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button1", button);
+  gtk_widget_grab_focus (button);
 
   button = gtk_check_button_new_with_label (_("Right"));
   if (editor_snap_to_grid_x & GB_SNAP_RIGHT)
@@ -473,7 +490,7 @@ editor_show_snap_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 1, 2, 1, 2,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button2", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button2", button);
 
   /* Vertical snapping */
   label = gtk_label_new (_("Vertical Snapping:"));
@@ -488,7 +505,7 @@ editor_show_snap_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 1, 2, 2, 3,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button3", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button3", button);
 
   button = gtk_check_button_new_with_label (_("Bottom"));
   if (editor_snap_to_grid_y & GB_SNAP_BOTTOM)
@@ -496,7 +513,7 @@ editor_show_snap_settings_dialog (void)
   gtk_table_attach (GTK_TABLE (table), button, 1, 2, 3, 4,
 		    GTK_EXPAND | GTK_FILL, 0, 5, 5);
   gtk_widget_show (button);
-  gtk_object_set_data (GTK_OBJECT (window), "button4", button);
+  gtk_object_set_data (GTK_OBJECT (dialog), "button4", button);
 
 
   separator = gtk_hseparator_new ();
@@ -531,34 +548,37 @@ editor_show_snap_settings_dialog (void)
   gtk_widget_show (button);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
+			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
 
-  gtk_widget_show (window);
+  gtk_widget_show (dialog);
 }
 
 
 static void
 on_snap_settings_ok (GtkWidget * widget, gpointer data)
 {
-  GtkWidget *window, *button;
+  GtkWidget *dialog, *button;
 
-  window = gtk_widget_get_toplevel (widget);
+  dialog = gtk_widget_get_toplevel (widget);
   editor_snap_to_grid_x = 0;
   editor_snap_to_grid_y = 0;
-  button = gtk_object_get_data (GTK_OBJECT (window), "button1");
+  button = gtk_object_get_data (GTK_OBJECT (dialog), "button1");
   if (GTK_TOGGLE_BUTTON (button)->active)
     editor_snap_to_grid_x |= GB_SNAP_LEFT;
-  button = gtk_object_get_data (GTK_OBJECT (window), "button2");
+  button = gtk_object_get_data (GTK_OBJECT (dialog), "button2");
   if (GTK_TOGGLE_BUTTON (button)->active)
     editor_snap_to_grid_x |= GB_SNAP_RIGHT;
-  button = gtk_object_get_data (GTK_OBJECT (window), "button3");
+  button = gtk_object_get_data (GTK_OBJECT (dialog), "button3");
   if (GTK_TOGGLE_BUTTON (button)->active)
     editor_snap_to_grid_y |= GB_SNAP_TOP;
-  button = gtk_object_get_data (GTK_OBJECT (window), "button4");
+  button = gtk_object_get_data (GTK_OBJECT (dialog), "button4");
   if (GTK_TOGGLE_BUTTON (button)->active)
     editor_snap_to_grid_y |= GB_SNAP_BOTTOM;
 
-  gtk_widget_destroy (window);
+  gtk_widget_destroy (dialog);
 }
 
 
@@ -1162,7 +1182,7 @@ static void
 add_widget_to_fixed_finish (GtkWidget * new_widget, GbWidgetNewData * data)
 {
   GtkRequisition requisition = {0, 0};
-  GbWidgetData *wdata;
+  GladeWidgetData *wdata;
   GtkWidget *parent = data->parent;
   gint x = data->x;
   gint y = data->y;
@@ -1228,7 +1248,7 @@ add_widget_to_fixed_finish (GtkWidget * new_widget, GbWidgetNewData * data)
      explicitly. */
   wdata = gtk_object_get_data (GTK_OBJECT (new_widget), GB_WIDGET_DATA_KEY);
   g_return_if_fail (wdata != NULL);
-  wdata->flags |= GB_X_SET | GB_Y_SET | GB_WIDTH_SET | GB_HEIGHT_SET;
+  wdata->flags |= GLADE_X_SET | GLADE_Y_SET | GLADE_WIDTH_SET | GLADE_HEIGHT_SET;
 
   /* Calculate real position & size according to grid & snapping */
   /* FIXME: only snaps top & left at present */
@@ -1507,7 +1527,7 @@ editor_select_widget (GtkWidget * widget, guint modifiers, gint x, gint y)
       if (found && ancestor->parent != NULL)
 	{
 	  select_widget = ancestor->parent;
-	  /* If widget is not a GbWidget, i.e. it has no GbWidgetData,
+	  /* If widget is not a GbWidget, i.e. it has no GladeWidgetData,
 	     skip it. */
 	  while (select_widget)
 	    {
@@ -1837,7 +1857,7 @@ static gint
 editor_do_drag_action (GtkWidget * signal_widget, GdkEventMotion * event)
 {
   GtkWidget *widget;
-  GbWidgetData *wdata;
+  GladeWidgetData *wdata;
   gint x, y, event_x, event_y;
   gint mouse_x, mouse_y, new_x = 0, new_y = 0, new_width = 0, new_height = 0;
   gint old_x, old_y;
@@ -2169,13 +2189,13 @@ editor_add_key_signals (GtkWidget * widget)
 void
 on_size_allocate (GtkWidget * widget,
 		  GtkAllocation *allocation,
-		  GbWidgetData * wdata)
+		  GladeWidgetData * wdata)
 {
   /* Reset the flag, since the size is allocated now. Note that wdata may be
      NULL as widget may be a placeholder. */
   if (wdata)
     {
-      wdata->flags &= ~GB_SIZE_NOT_ALLOCATED;
+      wdata->flags &= ~GLADE_SIZE_NOT_ALLOCATED;
     }
 
 #if 0
@@ -2192,7 +2212,7 @@ on_size_allocate (GtkWidget * widget,
 void
 editor_add_draw_signals (GtkWidget * widget)
 {
-  GbWidgetData *widget_data;
+  GladeWidgetData *widget_data;
 
   /* FIXME: Note that we set GDK_POINTER_MOTION_HINT_MASK here. This may not
      be wise since widgets may be designed to work with normal motion events
@@ -2232,7 +2252,7 @@ editor_add_draw_signals (GtkWidget * widget)
 
 static gint
 expose_widget (GtkWidget * widget, GdkEventExpose * event,
-	       GbWidgetData * widget_data)
+	       GladeWidgetData * widget_data)
 {
 #if 0
   g_print ("In expose_event widget:%s (%p)\n", gtk_widget_get_name (widget),
@@ -2706,6 +2726,7 @@ editor_on_key_press_event (GtkWidget * widget, GdkEventKey * event,
 
 	  toplevel = glade_util_get_toplevel (widget);
 	  glade_util_close_window (toplevel);
+	  gtk_widget_queue_resize (widget);
 	  gtk_widget_show (toplevel);
 	  handled = TRUE;
 	}
@@ -3046,6 +3067,13 @@ editor_delete_widget (GtkWidget * widget)
       return;
     }
 
+  /* If the widget is a menu item remove widget completely. */
+  if (GTK_IS_MENU_ITEM (widget))
+    {
+      gtk_widget_destroy (widget);
+      return;
+    }
+
 #ifdef USE_GNOME
   /* GnomeDockItem widgets are also removed completely rather than being
      replaced by a placeholder. */
@@ -3130,6 +3158,11 @@ editor_can_delete_widget (GtkWidget * widget)
   if (!GB_IS_PLACEHOLDER (widget) && !GB_IS_GB_WIDGET (widget))
     return _("The widget can't be deleted");
 
+  /* Non-toplevel menus are created automatically so we can't delete them. */
+  if (GTK_IS_MENU (widget) && gtk_menu_get_attach_widget (GTK_MENU (widget)))
+    return _("The widget can't be deleted");
+
+
   /* SPECIAL CODE: Don't allow dialog buttons & widgets to be deleted. */
   child_name = gb_widget_get_child_name (widget);
   if (child_name)
@@ -3162,14 +3195,16 @@ editor_can_delete_widget (GtkWidget * widget)
 	}
     }
 
-#ifdef USE_GNOME
-  /* GnomeDruid start and finish pages can't be deleted. */
-  if (GNOME_IS_DRUID_PAGE_START (widget)
-      || GNOME_IS_DRUID_PAGE_FINISH (widget))
-    {
-      return _("The start and finish pages of GnomeDruid widgets can't be deleted.");
-    }
-#endif
-
   return NULL;
 }
+
+
+/* Called when a GbWidget is destroyed so the editor can remove any references
+   to it. */
+void
+editor_on_widget_destroyed		(GtkWidget	    *widget)
+{
+  if (mouse_over_widget == widget)
+    mouse_over_widget = NULL;
+}
+
