@@ -39,7 +39,7 @@ gb_widget_write_source (GtkWidget * widget,
 {
   GtkWidget *parent;
   GbWidget *gbwidget;
-  GbWidgetData *widget_data;
+  GladeWidgetData *widget_data;
   gchar *class_id, *child_name;
   gint source_len;
 
@@ -205,7 +205,7 @@ void
 gb_widget_write_standard_source (GtkWidget * widget,
 				 GbWidgetWriteSourceData * data)
 {
-  GbWidgetData *wdata = data->widget_data;
+  GladeWidgetData *wdata = data->widget_data;
   gint i, width, height, can_focus, can_default;
 
   /* FIXME: unfinished. public fields were to be added to the component struct.
@@ -245,7 +245,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
     }
 
   /* SPECIAL CODE: menus are not shown here. */
-  if (widget->parent && wdata->flags & GB_VISIBLE && !GTK_IS_MENU (widget))
+  if (widget->parent && wdata->flags & GLADE_VISIBLE && !GTK_IS_MENU (widget))
     {
 #ifdef USE_GNOME
       /* FIXME: GnomeDruidPageStandard bug workaround. It needs show_all(). */
@@ -261,7 +261,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
   /* Output code to add widget to parent. */
   gb_widget_write_add_child_source (widget, data);
 
-  if (wdata->flags & (GB_X_SET | GB_Y_SET))
+  if (wdata->flags & (GLADE_X_SET | GLADE_Y_SET))
     {
       /* SPECIAL CODE: Widgets in a GtkLayout are positioned with
 	 gtk_layout_put(). */
@@ -269,15 +269,15 @@ gb_widget_write_standard_source (GtkWidget * widget,
 	{
 	  source_add (data, "  gtk_widget_set_uposition (%s, %i, %i);\n",
 		      data->wname,
-		      wdata->flags & GB_X_SET ? wdata->x : -2,
-		      wdata->flags & GB_Y_SET ? wdata->y : -2);
+		      wdata->flags & GLADE_X_SET ? wdata->x : -2,
+		      wdata->flags & GLADE_Y_SET ? wdata->y : -2);
 	}
     }
 
-  if (wdata->flags & (GB_WIDTH_SET | GB_HEIGHT_SET))
+  if (wdata->flags & (GLADE_WIDTH_SET | GLADE_HEIGHT_SET))
     {
-      width = wdata->flags & GB_WIDTH_SET ? wdata->width : -2;
-      height = wdata->flags & GB_HEIGHT_SET ? wdata->height : -2;
+      width = wdata->flags & GLADE_WIDTH_SET ? wdata->width : -2;
+      height = wdata->flags & GLADE_HEIGHT_SET ? wdata->height : -2;
 
       /* GTK BUG WORKAROUND - a combo should manage the size of its entry.
 	 I think this isn't needed any more (GTK+ 1.2.3). */
@@ -306,7 +306,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
 
   /* FIXME: Kludge to set separator menu items insensitive, so that they are
      skipped when using the cursor keys to move up/down the menu. */
-  if (!(wdata->flags & GB_SENSITIVE)
+  if (!(wdata->flags & GLADE_SENSITIVE)
       || (GTK_IS_MENU_ITEM (widget) && GTK_BIN (widget)->child == NULL))
     source_add (data, "  gtk_widget_set_sensitive (%s, FALSE);\n", data->wname);
 
@@ -334,7 +334,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
 		    data->wname);
     }
 
-  if (wdata->flags & GB_GRAB_FOCUS)
+  if (wdata->flags & GLADE_GRAB_FOCUS)
     {
       if (data->focus_widget)
 	g_warning ("Multiple widgets with 'Has Focus' set: %s, %s",
@@ -343,7 +343,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
 	data->focus_widget = g_strdup (data->wname);
     }
 
-  if (wdata->flags & GB_GRAB_DEFAULT)
+  if (wdata->flags & GLADE_GRAB_DEFAULT)
     {
       if (data->default_widget)
 	g_warning ("Multiple widgets with 'Has Default' set: %s, %s",
@@ -450,14 +450,14 @@ gb_widget_write_signals_source (GtkWidget * widget,
 				GbWidgetWriteSourceData * data)
 {
   GList *item;
-  GbSignal *signal;
+  GladeSignal *signal;
   gboolean skip_handler_code;
   gchar *handler;
 
   item = data->widget_data->signals;
   while (item)
     {
-      signal = (GbSignal *) item->data;
+      signal = (GladeSignal *) item->data;
       item = item->next;
       skip_handler_code = FALSE;
 
@@ -501,7 +501,9 @@ gb_widget_write_signals_source (GtkWidget * widget,
       /* We don't need to output code for standard GTK functions. */
       if (!strcmp (handler, "gtk_widget_show")
 	  || !strcmp (handler, "gtk_widget_hide")
+	  || !strcmp (handler, "gtk_widget_grab_focus")
 	  || !strcmp (handler, "gtk_widget_destroy")
+	  || !strcmp (handler, "gtk_window_activate_default")
 	  || !strcmp (handler, "gtk_true")
 	  || !strcmp (handler, "gtk_false")
 	  || !strcmp (handler, "gtk_main_quit"))
@@ -1089,11 +1091,11 @@ gb_widget_write_accelerators_source (GtkWidget * widget,
 				     GbWidgetWriteSourceData * data)
 {
   GList *item = data->widget_data->accelerators;
-  GbAccelerator *accel;
+  GladeAccelerator *accel;
 
   while (item)
     {
-      accel = (GbAccelerator *) item->data;
+      accel = (GladeAccelerator *) item->data;
       item = item->next;
 
       /* The code to create the accel_group is output in source.c */
