@@ -54,12 +54,38 @@ static gchar *Label = "GtkButton::label";
 /* This is only used for toolbar buttons. */
 static gchar *Icon = "GtkButton::icon";
 
+/* This is only used for normal/stock buttons, not toolbar buttons or Gnome
+   dialog buttons. */
+static gchar *Relief = "GtkButton::relief";
 
 #ifdef USE_GNOME
 /* This is used for Gnome Stock buttons. */
 static gchar *StockButton = "GtkButton::stock_button";
 static gchar *StockIcon = "GtkButton::stock_pixmap";
+#endif
 
+static const gchar *GbReliefChoices[] =
+{
+  "Normal",
+  "Half",
+  "None",
+  NULL
+};
+static const gint GbReliefValues[] =
+{
+  GTK_RELIEF_NORMAL,
+  GTK_RELIEF_HALF,
+  GTK_RELIEF_NONE
+};
+static const gchar *GbReliefSymbols[] =
+{
+  "GTK_RELIEF_NORMAL",
+  "GTK_RELIEF_HALF",
+  "GTK_RELIEF_NONE"
+};
+
+
+#ifdef USE_GNOME
 static void gb_button_output_gnome_dialog_button_label (GtkWidget * widget,
 							GbWidgetGetArgData * data,
 							const gchar * Label,
@@ -141,11 +167,15 @@ gb_button_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 		       _("The Gnome stock button to use"),
 		       GladeStockButtonChoices);
   property_add_text (Label, _("Label:"), _("The text to display"), 2);
-
+  property_add_choice (Relief, _("Button Relief:"),
+		       _("The relief style of the button"),
+		       GbReliefChoices);
   gb_button_create_child_icon_property (widget, data, Icon);
-
 #else
   property_add_text (Label, _("Label:"), _("The text to display"), 2);
+  property_add_choice (Relief, _("Button Relief:"),
+		       _("The relief style of the button"),
+		       GbReliefChoices);
   property_add_filename (Icon, _("Icon:"), _("The pixmap to display"));
 #endif
 }
@@ -207,6 +237,9 @@ gb_button_create_child_icon_property (GtkWidget * widget,
 static void
 gb_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 {
+  gboolean relief_visible = FALSE;
+  gint i;
+
 #ifdef USE_GNOME
   gint stock_button_index;
   gboolean stock_item_visible;
@@ -233,6 +266,7 @@ gb_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 	}
       else
 	{
+	  relief_visible = TRUE;
 	  gb_widget_output_child_label (widget, data, Label);
 
 	  if (data->action == GB_SHOWING)
@@ -253,9 +287,22 @@ gb_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
     }
   else
     {
+      relief_visible = TRUE;
       gb_widget_output_child_label (widget, data, Label);
     }
 #endif
+
+  if (data->action == GB_SHOWING)
+    property_set_visible (Relief, relief_visible);
+
+  if (relief_visible)
+    {
+      for (i = 0; i < sizeof (GbReliefValues) / sizeof (GbReliefValues[0]); i++)
+	{
+	  if (GbReliefValues[i] == GTK_BUTTON (widget)->relief)
+	    gb_widget_output_choice (data, Relief, i, GbReliefSymbols[i]);
+	}
+    }
 }
 
 
@@ -266,6 +313,10 @@ gb_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 static void
 gb_button_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
+  gboolean set_relief = FALSE;
+  gchar *relief;
+  gint i;
+
 #ifdef USE_GNOME
   gchar *stock_button;
   gboolean status;
@@ -296,6 +347,7 @@ gb_button_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 	}
       else
 	{
+	  set_relief = TRUE;
 	  gb_widget_input_child_label (widget, data, Label);
 	}
     }
@@ -308,9 +360,29 @@ gb_button_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
     }
   else
     {
+      set_relief = TRUE;
       gb_widget_input_child_label (widget, data, Label);
     }
 #endif
+
+  if (set_relief)
+    {
+      relief = gb_widget_input_choice (data, Relief);
+      if (data->apply)
+	{
+	  for (i = 0; i < sizeof (GbReliefValues) / sizeof (GbReliefValues[0]);
+	       i++)
+	    {
+	      if (!strcmp (relief, GbReliefChoices[i])
+		  || !strcmp (relief, GbReliefSymbols[i]))
+		{
+		  gtk_button_set_relief (GTK_BUTTON (widget),
+					 GbReliefValues[i]);
+		  break;
+		}
+	    }
+	}
+    }
 }
 
 
@@ -341,6 +413,8 @@ gb_button_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
   GtkWidget *child = GTK_BUTTON (widget)->child;
   gchar *label_text;
+  gboolean set_relief = TRUE;
+  gint i;
 
 #ifdef USE_GNOME
   if (glade_gnome_is_gnome_dialog_button (widget))
@@ -359,6 +433,7 @@ gb_button_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
       if (gb_toolbar_is_toolbar_button (widget))
 	{
 	  gb_toolbar_write_toolbar_button_source (widget, data);
+	  set_relief = FALSE;
 	}
 #ifdef USE_GNOME
       else if (stock_button_index > 0)
@@ -392,6 +467,18 @@ gb_button_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
     }
 
   gb_widget_write_standard_source (widget, data);
+
+  if (set_relief && GTK_BUTTON (widget)->relief != GTK_RELIEF_NORMAL)
+    {
+      for (i = 0; i < sizeof (GbReliefValues) / sizeof (GbReliefValues[0]);
+	   i++)
+	{
+	  if (GbReliefValues[i] == GTK_BUTTON (widget)->relief)
+	    source_add (data,
+			"  gtk_button_set_relief (GTK_BUTTON (%s), %s);\n",
+			data->wname, GbReliefSymbols[i]);
+	}
+    }
 }
 
 
@@ -880,7 +967,7 @@ gb_button_write_gnome_dialog_button_source (GtkWidget * widget,
      We can just get the last widget in the GnomeDialog's list of buttons.
      Note that we have to make sure that the button isn't added to its
      parent widget again elsewhere. */
-  source_add (data, "  %s = g_list_last (GNOME_DIALOG (%s)->buttons)->data;\n",
+  source_add (data, "  %s = GTK_WIDGET (g_list_last (GNOME_DIALOG (%s)->buttons)->data);\n",
 	      data->wname, data->component_name);
 
   /* We set this to FALSE, so the code to add it to its parent is skipped. */
