@@ -25,6 +25,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtklist.h>
 #include <gtk/gtklistitem.h>
+#include <gtk/gtkeventbox.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -93,7 +94,21 @@ static void gb_label_get_focus_targets (GtkWidget * widget,
 GtkWidget *
 gb_label_new (GbWidgetNewData * data)
 {
-  return gtk_label_new (data->name);
+  GtkWidget *new_widget;
+
+  new_widget = gtk_label_new (data->name);
+
+  /* If we are creating a new label in a table or in an event box in a table
+     set it to left-aligned, since that is what is usually wanted. */
+  if (data->action == GB_CREATING && data->parent
+      && (GTK_IS_TABLE (data->parent)
+	  || (GTK_IS_EVENT_BOX (data->parent) && data->parent->parent
+	      && GTK_IS_TABLE (data->parent->parent))))
+    {
+      gtk_misc_set_alignment (GTK_MISC (new_widget), 0.0, 0.5);
+    }
+
+  return new_widget;
 }
 
 
