@@ -131,6 +131,24 @@ gb_menu_bar_create_properties(GtkWidget *widget, GbWidgetCreateArgData *data)
 }
 
 
+/* Make window behave like a dialog */
+static void
+dialogize (GtkWidget *menued, GtkWidget *parent_widget)
+{
+  GtkWidget *transient_parent;
+
+  GTK_WINDOW(menued)->type = GTK_WINDOW_DIALOG;
+  gtk_signal_connect (GTK_OBJECT (menued), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
+  transient_parent = glade_util_get_toplevel (parent_widget);
+  if (GTK_IS_WINDOW (transient_parent))
+    {
+      gtk_window_set_transient_for (GTK_WINDOW (menued),
+				    GTK_WINDOW (transient_parent));
+    }
+}
+
 static void
 gb_menu_bar_on_edit_menu (GtkWidget *button,
 			  gpointer data)
@@ -141,6 +159,7 @@ gb_menu_bar_on_edit_menu (GtkWidget *button,
   g_return_if_fail (GTK_IS_MENU_BAR (menubar));
 
   menued = glade_menu_editor_new (current_project, GTK_MENU_SHELL (menubar));
+  dialogize (menued, button);
   gtk_widget_show (GTK_WIDGET (menued));
 }
 
@@ -218,6 +237,7 @@ gb_menu_bar_on_edit_menu_activate (GtkWidget *menuitem,
   GtkWidget *menued;
 
   menued = glade_menu_editor_new (current_project, GTK_MENU_SHELL (menubar));
+  dialogize (menued, menubar);
   gtk_widget_show (GTK_WIDGET (menued));
 }
 
