@@ -252,7 +252,7 @@ gb_menu_item_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 	  tmp_list = data->accelerators;
 	  while (tmp_list)
 	    {
-	      GbAccelerator *accel = (GbAccelerator *) tmp_list->data;
+	      GladeAccelerator *accel = (GladeAccelerator *) tmp_list->data;
 	      if (!strcmp (accel->signal, "activate")
 		  && widget->parent && GTK_IS_MENU (widget->parent))
 		{
