@@ -228,14 +228,6 @@ gb_pixmap_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
   if (data->create_widget)
     {
       gchar *filename = gtk_object_get_data (GTK_OBJECT (widget), GbFilenameKey);
-      /* If the pixmap filename hasn't been set, we simply return, and hope
-	 it doesn't affect the rest of the application. */
-      if (!filename || filename[0] == '\0')
-	{
-	  MSG ("Pixmap filename is not set");
-	  return;
-	}
-
       source_create_pixmap (data, data->wname, filename, FALSE);
     }
 
