@@ -240,9 +240,8 @@ show_toolbar_dialog (GbWidgetNewData * data)
   gtk_widget_show (label);
 
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
-  /* save pointer to spinbutton so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (dialog), "items", spinbutton);
+  spinbutton = glade_util_spin_button_new (GTK_OBJECT (dialog), "items",
+					   GTK_ADJUSTMENT (adjustment), 1, 0);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, TRUE, 5);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_grab_focus (spinbutton);
@@ -840,7 +839,7 @@ void
 gb_toolbar_write_toolbar_button_source (GtkWidget * widget,
 				       GbWidgetWriteSourceData * data)
 {
-  GbWidgetData *wdata = data->widget_data;
+  GladeWidgetData *wdata = data->widget_data;
   GtkWidget *icon, *label, *radio_widget;
   gchar *parent_name, *label_text, *type, *radio_widget_name = NULL;
   gboolean has_icon;
@@ -1582,6 +1581,11 @@ gb_toolbar_write_add_child_source (GtkWidget * parent,
      created, so we skip this. For standard widgets, we add them here. */
   if (!gb_toolbar_is_toolbar_button (child))
     {
+      if (gb_toolbar_get_new_toolbar_group (parent, child))
+	source_add (data,
+		    "  gtk_toolbar_append_space (GTK_TOOLBAR (%s));\n\n",
+		    parent_name);
+
       source_add (data,
 		  "  gtk_toolbar_append_widget (GTK_TOOLBAR (%s), %s, %s, NULL);\n",
 		  parent_name, data->wname,
