@@ -119,9 +119,8 @@ show_vbbox_dialog (GbWidgetNewData * data)
   gtk_widget_show (label);
 
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
-  /* save pointer to spinbutton so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (dialog), "rows", spinbutton);
+  spinbutton = glade_util_spin_button_new (GTK_OBJECT (dialog), "rows",
+					   GTK_ADJUSTMENT (adjustment), 1, 0);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, TRUE, 5);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_grab_focus (spinbutton);
