@@ -738,6 +738,10 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_signal_connect_object (GTK_OBJECT (options->cancel_button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (options));
+  gtk_signal_connect (GTK_OBJECT (options), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
+  gtk_widget_grab_focus (options->directory_entry);
 }
 
 
@@ -1165,6 +1169,9 @@ glade_project_options_show_file_selection (GtkWidget *widget,
       gtk_signal_connect (GTK_OBJECT (options->filesel), "delete_event",
 			  GTK_SIGNAL_FUNC (glade_util_close_window_on_delete),
 			  NULL);
+      gtk_signal_connect (GTK_OBJECT (options->filesel), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscCloses));
     }
 
   filename = gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
@@ -1173,6 +1180,9 @@ glade_project_options_show_file_selection (GtkWidget *widget,
       gtk_file_selection_set_filename (GTK_FILE_SELECTION (options->filesel),
 				       filename);
     }
+  if (GTK_IS_WINDOW (options))
+    gtk_window_set_transient_for (GTK_WINDOW (options->filesel),
+				  GTK_WINDOW (options));
   gtk_widget_show (options->filesel);
   gdk_window_show (options->filesel->window);
   gdk_window_raise (options->filesel->window);
@@ -1362,8 +1372,10 @@ glade_project_options_check_valid	(GladeProjectOptions *options)
 	error = _("You need to set the Project Name option");
       else if (program_name == NULL || program_name[0] == '\0')
 	error = _("You need to set the Program Name option");
+#if 0
       else if (source_directory == NULL || source_directory[0] == '\0')
 	error = _("You need to set the Source Directory option");
+#endif
       else if (pixmaps_directory == NULL || pixmaps_directory[0] == '\0')
 	error = _("You need to set the Pixmaps Directory option");
       break;
