@@ -876,6 +876,9 @@ glade_project_window_on_open_project (GtkWidget *widget,
   filesel = gtk_file_selection_new (_("Open Project"));
   gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
   gtk_window_set_wmclass (GTK_WINDOW (filesel), "open_project", "Glade");  
+  if (GTK_IS_WINDOW (project_window->window))
+    gtk_window_set_transient_for (GTK_WINDOW (filesel),
+				  GTK_WINDOW (project_window->window));
   if (project_window->current_directory)
     gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
 				     project_window->current_directory);
@@ -887,6 +890,9 @@ glade_project_window_on_open_project (GtkWidget *widget,
   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
 			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (filesel));
+  gtk_signal_connect (GTK_OBJECT (filesel), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
   gtk_widget_show (filesel);
 }
 
@@ -1020,6 +1026,9 @@ glade_project_window_new_errors_dialog (GladeProjectWindow *project_window,
   gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
 
   *text_return = text;
   return dialog;
@@ -1061,6 +1070,10 @@ glade_project_window_edit_options (GladeProjectWindow *project_window,
       if (action)
 	glade_project_options_set_action (GLADE_PROJECT_OPTIONS (options),
 					  action);
+      if (GTK_IS_WINDOW (project_window->window))
+	gtk_window_set_transient_for (GTK_WINDOW (options),
+				      GTK_WINDOW (project_window->window));
+      				      
       gtk_widget_show (options);
     }
 }
@@ -1146,6 +1159,10 @@ glade_project_window_save_project_as (GladeProjectWindow *project_window)
 
   filesel = gtk_file_selection_new (_("Save Project"));
   gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW (filesel), "save_project", "Glade");  
+  if (GTK_IS_WINDOW (project_window->window))
+    gtk_window_set_transient_for (GTK_WINDOW (filesel),
+				  GTK_WINDOW (project_window->window));
   if (project_window->current_directory)
     gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
 				     project_window->current_directory);
@@ -1156,6 +1173,9 @@ glade_project_window_save_project_as (GladeProjectWindow *project_window)
   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
 			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (filesel));
+  gtk_signal_connect (GTK_OBJECT (filesel), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
   gtk_widget_show (filesel);
 }
 
@@ -1304,6 +1324,9 @@ glade_project_window_show_error (GladeProjectWindow *project_window,
 
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_window_set_wmclass (GTK_WINDOW (dialog), "error", "Glade");
+  if (GTK_IS_WINDOW (project_window->window))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (project_window->window));
 
   gtk_text_freeze (GTK_TEXT (text));
 
@@ -1510,7 +1533,7 @@ static void
 glade_project_window_edit_grid_settings (GtkWidget *widget,
 					 gpointer   data)
 {
-  editor_show_grid_settings_dialog ();
+  editor_show_grid_settings_dialog (widget);
 }
 
 
@@ -1529,7 +1552,7 @@ static void
 glade_project_window_edit_snap_settings (GtkWidget *widget,
 					 gpointer   data)
 {
-  editor_show_snap_settings_dialog ();
+  editor_show_snap_settings_dialog (widget);
 }
 
 
@@ -1539,7 +1562,7 @@ glade_project_window_about (GtkWidget *widget,
 {
 #ifdef USE_GNOME
   const gchar *author[] = {"Damon Chaplin <glade@glade.pn.org>", NULL };
-  GtkWidget *about;
+  GtkWidget *about, *transient_parent;
 #else
   gchar *message;
 #endif
@@ -1554,6 +1577,10 @@ glade_project_window_about (GtkWidget *widget,
 			   PACKAGE G_DIR_SEPARATOR_S "glade_logo.png");
   gtk_window_set_modal (GTK_WINDOW (about), TRUE);
   gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
+  transient_parent = glade_util_get_toplevel (widget);
+  if (GTK_IS_WINDOW (transient_parent))
+    gtk_window_set_transient_for (GTK_WINDOW (about),
+				  GTK_WINDOW (transient_parent));
   gtk_widget_show (about);
 #else
   message = g_strdup_printf (_("G L A D E\n\n"
