@@ -75,7 +75,7 @@ gb_gnome_animator_new (GbWidgetNewData *data)
   new_widget = gnome_animator_new_with_size (100, 100);
 
   /* GnomeAnimator widgets must have a width & height so we set the flags. */
-  data->widget_data->flags |= GB_WIDTH_SET | GB_HEIGHT_SET;
+  data->widget_data->flags |= GLADE_WIDTH_SET | GLADE_HEIGHT_SET;
 
   return new_widget;
 }
