@@ -165,7 +165,7 @@ real_load_project_file (GladeProject *project,
 
       if (data.token_type == GB_TOKEN_END_TAG)
 	{
-	  if (!strcmp (data.buffer.ptr + data.token, "gtk-interface"))
+	  if (!strcmp (data.buffer.ptr + data.token, "GTK-Interface"))
 	    {
 	      MSG1 ("End of Document body element: %s",
 		    data.buffer.ptr + data.token);
@@ -199,7 +199,7 @@ real_load_project_file (GladeProject *project,
 	  if (data.status != GLADE_STATUS_OK)
 	    break;
 	}
-      else if (!strcmp (data.buffer.ptr + data.token, "gtk-interface"))
+      else if (!strcmp (data.buffer.ptr + data.token, "GTK-Interface"))
 	{
 	  MSG1 ("Document body element: %s", data.buffer.ptr + data.token);
 	}
@@ -301,7 +301,7 @@ load_element (GbWidgetSetArgData * data, gint * element, gint * cdata)
 
 
 /* Returns one token of XML - a GB_TOKEN_START_TAG, GB_TOKEN_END_TAG or
-   GB_TOKEN_DATA. Start and end tags are converted to lower case. */
+   GB_TOKEN_DATA. */
 void
 load_token (GbWidgetSetArgData * data)
 {
@@ -344,7 +344,6 @@ load_token (GbWidgetSetArgData * data)
 	  if (ch == '>')
 	    {
 	      load_buffer_add_char (&data->buffer, '\0');
-	      g_strdown (data->buffer.ptr + data->token);
 	      return;
 	    }
 	  else
