@@ -29,6 +29,7 @@
 #include <errno.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "gladeconfig.h"
 
@@ -98,14 +99,18 @@ static void glade_util_find_widget_recursive (GtkWidget *widget,
 /* This shows a simple dialog box with a label and an 'OK' button.
    Example usage:
     glade_util_show_message_box ("Error saving file");
- */
+
+    FIXME: NO WAY TO SET THE TRANSIENT PARENT
+*/
 void
 glade_util_show_message_box (const gchar * message)
 {
 #ifdef USE_GNOME
-  GtkWidget *dialog;
+  GtkWidget *dialog, *button;
   dialog = gnome_message_box_new (message, GNOME_MESSAGE_BOX_ERROR,
 				  GNOME_STOCK_BUTTON_OK, NULL);
+  button = GNOME_DIALOG (dialog)->buttons->data;
+  gtk_widget_grab_focus (button);
   gtk_widget_show (dialog);
 #else
   GtkWidget *dialog, *label, *button;
@@ -126,11 +131,15 @@ glade_util_show_message_box (const gchar * message)
 		      FALSE, FALSE, 14);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
+  gtk_widget_grab_focus (button);
   gtk_widget_show (button);
 
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
   gtk_widget_show (dialog);
 #endif
 }
@@ -187,8 +196,10 @@ glade_util_create_dialog_with_buttons (const gchar * message,
       button = gtk_button_new_with_label (_(buttons[i]));
       gtk_container_add (GTK_CONTAINER (bbox), button);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-      if (i == default_button - 1)
+      if (i == default_button - 1) {
 	gtk_widget_grab_default (button);
+	gtk_widget_grab_focus (button);
+      }
       gtk_widget_show (button);
 
       if (signal_handlers[i])
@@ -198,6 +209,9 @@ glade_util_create_dialog_with_buttons (const gchar * message,
       gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 				 GTK_SIGNAL_FUNC (gtk_widget_destroy),
 				 GTK_OBJECT (dialog));
+      gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+			  GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+			  GINT_TO_POINTER (GladeEscDestroys));
     }
   return dialog;
 }
@@ -216,7 +230,9 @@ glade_util_create_dialog_with_buttons (const gchar * message,
      on_dialog_ok(GtkWidget *widget, gchar *value, gpointer data)
      {
      ...
- */
+
+     FIXME: NO WAY TO SET THE TRANSIENT PARENT (?)
+*/
 void
 glade_util_show_entry_dialog (const gchar * message,
 			      const gchar * initial_value,
@@ -245,7 +261,7 @@ glade_util_show_entry_dialog (const gchar * message,
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
 
-  entry = gtk_entry_new ();
+  entry = glade_util_entry_new (GTK_OBJECT (dialog));
   if (initial_value)
     gtk_entry_set_text (GTK_ENTRY (entry), initial_value);
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
@@ -271,6 +287,9 @@ glade_util_show_entry_dialog (const gchar * message,
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      GTK_SIGNAL_FUNC (on_entry_dialog_destroy), widget);
   gtk_widget_show (dialog);
@@ -359,6 +378,9 @@ glade_util_create_dialog	(const gchar	   *title,
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (dialog));
+  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
+		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
+		      GINT_TO_POINTER (GladeEscDestroys));
   *vbox = GTK_DIALOG (dialog)->vbox;
 #endif
 
@@ -374,6 +396,34 @@ glade_util_create_dialog	(const gchar	   *title,
 }
 
 
+gint glade_util_check_key_is_esc (GtkWidget *widget,
+				  GdkEventKey *event,
+				  gpointer data)
+{
+  g_return_val_if_fail (GTK_IS_WINDOW (widget), FALSE);
+  
+  if (event->keyval == GDK_Escape)
+    {
+      GladeEscAction action = GPOINTER_TO_INT (data);
+  
+      if (action == GladeEscCloses)
+        {
+  	glade_util_close_window (widget);
+  	return TRUE;
+        }
+      else if (action == GladeEscDestroys)
+        { 
+  	gtk_widget_destroy (widget);
+  	return TRUE;
+        }
+      else
+        return FALSE;
+    }
+  else
+    return FALSE;
+}
+
+
 #ifdef USE_GNOME
 static void gnome_dialog_button_clicked (GnomeDialog *dialog,
 					 gint button,
@@ -393,7 +443,58 @@ static void gnome_dialog_button_clicked (GnomeDialog *dialog,
 }
 #endif /* USE_GNOME */
 
+/* This returns a new entry ready to insert in a dialog.
+   The entry is set up so that <Return> will invoke the default action. The
+   returned widget must be added to a container in the dialog. */ 
+extern GtkWidget *
+glade_util_entry_new (GtkObject *dialog)
+{
+  GtkWidget *entry;
 
+  g_return_val_if_fail (GTK_IS_WINDOW (dialog), NULL);
+
+  entry = gtk_entry_new ();
+  g_return_val_if_fail (entry != NULL, NULL);
+  
+  /* Make <Return> in entry field invoke dialog default */
+  gtk_signal_connect_object (GTK_OBJECT (entry), "activate",
+			     GTK_SIGNAL_FUNC (gtk_window_activate_default), 
+			     GTK_OBJECT (dialog));
+
+  return entry;
+}
+
+
+/* This returns a new spinbutton ready to insert in a dialog.
+   A pointer to the spin button is added as object data to the dialog. The
+   spinbutton is set up so that <Return> will invoke the default action. The
+   returned widget must be added to a container in the dialog. */ 
+GtkWidget *
+glade_util_spin_button_new (GtkObject *dialog,
+			    const gchar *key,
+			    GtkAdjustment *adjustment,
+			    gfloat climb_rate,
+			    guint digits)
+{
+  GtkWidget *spinbutton;
+
+  g_return_val_if_fail (GTK_IS_WINDOW (dialog), NULL);
+  g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);
+
+  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), climb_rate,
+				    digits);
+  g_return_val_if_fail (spinbutton != NULL, NULL);
+  
+  /* save pointer to entry so we can find it easily in the OK handler */
+  gtk_object_set_data (GTK_OBJECT (dialog), key, spinbutton);
+  /* Make <Return> in entry field invoke dialog default */
+  gtk_signal_connect_object (GTK_OBJECT (spinbutton), "activate",
+			     GTK_SIGNAL_FUNC (gtk_window_activate_default), 
+			     GTK_OBJECT (dialog));
+
+  return spinbutton;
+}
+			    
 /* This returns the index of the given gint with the given array of gints.
    If the value is not found it outputs a warning and returns -1.
    This function is intended to be used for finding the index when using
@@ -1277,7 +1378,11 @@ glade_util_ensure_directory_exists (const gchar *directory)
 	  if (mkdir (directory, 0777) != 0)
 #endif
 	    {
+#ifndef _WIN32
+/* This happens under WIN32 when stat is confused by the filename, but this is
+   harmless, since we know that the directory exists after all. */
 	      return glade_error_new_system (_("Couldn't create directory:\n  %s\n"), directory);
+#endif
 	    }
 	}
       else
